/****************************************************************************
MOUSE CONTROL FUNCTIONS

Copyright (c) Gary Wood All Rights Reserved
Not for Commercial or Business Use without permission of Gary Wood,
5401 Lakeview Road, North Little Rock, Arkansas  72116

Use the /DOS2 option to set the MSC 6.0 compiler to use OS/2 calls

****************************************************************************/

#ifndef OS2
#include <dos.h>
#include <bios.h>
#endif
#define MOUHDRS 1
#include <stdlib.h>
#include <malloc.h>
#include <mouse.h>

#define ON    1
#define OFF   0
#define BW40  0
#define CO40  1
#define BW80  2
#define CO80  3
#define MONO  7

static int MOUSE_V_MODE = 3;

/****************************************************************************/
/*  Initialize Mouse                                                        */
/*                                                                          */
/*  Call:  mouse_open(row,col,mask,wait,click)                              */
/*         row = integer initial row position of cursor                     */
/*         col = integer initial column position of cursor                  */
/*         mask = what functions of mouse to support (see OS/2 Technical    */
/*                Reference for MouSetEventMask()                           */
/*         wait =                                                           */
/*         click =                                                          */
/*                                                                          */
/*  Returns:  OS/2 = handle (0xff = not functional)                         */
/*                                                                          */
/****************************************************************************/
int _fastcall mouse_open(row,col,mask,wait,click)
int row,col;
unsigned int mask;
int wait;
int click;
{
#ifdef OS2
    VIOMODEINFO ModData;

    MOUSE_HANDLE = 0xff;
/*      ModData.cb = sizeof(VIOMODEINFO);
        VioGetMode((VIOMODEINFO far *) &ModData,0);
        if (ModData.col == 40)
                if (ModData.fbType & 0x01)
                        MOUSE_V_MODE = CO40;
                else
                        MOUSE_V_MODE = BW40;
        else
        if (ModData.col == 80)
                if (ModData.fbType & 0x01)
                        MOUSE_V_MODE = CO80;
                else
                        MOUSE_V_MODE = BW80;
        else
                MOUSE_V_MODE = CO80;
*/
    MouOpen(0L, (unsigned far *) &MOUSE_HANDLE);
    if (MOUSE_HANDLE != 0xff)
    {
	MOUSE_POS.row = row;                 /* Second Row      */
	MOUSE_POS.col = col;                 /* Column position */
	MouSetPtrPos((PTRLOC far *) &MOUSE_POS, MOUSE_HANDLE);
	MOUSE_EVENT_MASK = mask;             /* Set mask        */
	MouSetEventMask((unsigned far *) &MOUSE_EVENT_MASK, MOUSE_HANDLE);
	MOUSE_READ_TYPE = wait;
	MOUSE_STATE_PTR = NULL;
    }
#else
    union REGS xregs;

    MOUSE_HANDLE = 0xff;
    if (_dos_getvect(0x33))
    {
	xregs.x.ax = 0x0F00;
	int86(0x10,&xregs,&xregs);
	MOUSE_V_MODE = xregs.x.ax & 0xFF;

	xregs.x.ax = 0x00;
	int86(0x33,&xregs,&xregs);
	if (xregs.x.ax == -1)
	{
	    xregs.x.ax = 0x04;
	    xregs.x.cx = col * 8;
	    xregs.x.dx = row * 8;
	    int86(0x33,&xregs,&xregs);
	    xregs.x.ax = 0x000a;
	    xregs.x.bx = 0x0000;
	    xregs.x.cx = 0x77ff;
	    xregs.x.dx = 0x7700;
	    int86(0x33,&xregs,&xregs); /* set shape */
	    MOUSE_HANDLE = 0x00;
	}
    }
#endif

    MOUSE_CLICK_SWT = click;
    MOUSE_EVENT_BUFFER.time = 0L;
    MOUSE_EVENT_BUFFER.fs   = 0;
    MOUSE_EVENT_BUFFER.col  = col;
    MOUSE_EVENT_BUFFER.row  = row;
    MOUSE_HIDE_COUNT        = 1;

    return(MOUSE_HANDLE);
}
/****************************************************************************/
/*  WAIT FOR KEYSTROKE                                                      */
/*                                                                          */
/*  Call:  mouse_wait_ki()                                                  */
/*                                                                          */
/*  Returns:  0 = Mouse button not pressed before keystroke                 */
/*            0x04 = Button 1 Pressed                                       */
/*            0x10 = Button 2 Pressed                                       */
/*            0x40 = Button 3 Pressed                                       */
/*                                                                          */
/****************************************************************************/
int _fastcall mouse_wait_ki()
{
    int fs;

    if (MOUSE_HANDLE != 0xff)
    {
	while (!mouse_ki_chk())
	{
	    if ((fs = mouse_read(ON)))
		if (MOUSE_EVENT_BUFFER.fs & MOUSE_BN1_DOWN ||
		    MOUSE_EVENT_BUFFER.fs & MOUSE_BN2_DOWN ||
		    MOUSE_EVENT_BUFFER.fs & MOUSE_BN3_DOWN)
		    return(fs);
	}
    } else
	while(!mouse_ki_chk()) ;

    return(OFF);
}

/****************************************************************************/
/*  READ  MOUSE                                                             */
/*                                                                          */
/*  Call:  mouse_read(flag)                                                 */
/*         flag = Wait for mouse press when = 1                             */
/*                                                                          */
/*  Will set the information in structure MOUEVENTINFO to reflect           */
/*  the button press, row, and column location of cursor. (see mouse.h)     */
/*                                                                          */
/*  Returns:  0x00 = No Buttons Pressed                                     */
/*            0x04 = Button 1 Pressed                                       */
/*            0x10 = Button 2 Pressed                                       */
/*            0x40 = Button 3 Pressed                                       */
/*                                                                          */
/****************************************************************************/
int _fastcall mouse_read(flag)
int flag;
{
#ifdef OS2
    MOUEVENTINFO work_event_buffer;   /* Mouse event buffer */
#else
    union REGS xregs;
#endif

    if (MOUSE_HANDLE == 0xff) return(-1);
#ifdef OS2
    MouReadEventQue((MOUEVENTINFO far *) &work_event_buffer,
       (unsigned far *) &MOUSE_READ_TYPE,MOUSE_HANDLE);
    if (work_event_buffer.time)
    {
	MOUSE_EVENT_BUFFER.time = work_event_buffer.time;
	MOUSE_EVENT_BUFFER.fs = work_event_buffer.fs;
	if (work_event_buffer.fs) MOUSE_SAVE_FS = work_event_buffer.fs;
	MOUSE_EVENT_BUFFER.col = work_event_buffer.col;
	MOUSE_EVENT_BUFFER.row = work_event_buffer.row;
    }
#else
    MOUSE_EVENT_BUFFER.fs = 0;
    xregs.x.ax = 0x03;
    int86(0x33,&xregs,&xregs);
    if (xregs.x.bx)
    {
	if (xregs.x.bx & 0x0001)        /* Button 1 */
	    MOUSE_EVENT_BUFFER.fs = MOUSE_EVENT_BUFFER.fs | MOUSE_BN1_DOWN;
	if (xregs.x.bx & 0x0002)        /* Button 2 */
	    MOUSE_EVENT_BUFFER.fs = MOUSE_EVENT_BUFFER.fs | MOUSE_BN2_DOWN;
	if (xregs.x.bx & 0x0004)        /* Button 3 */
	    MOUSE_EVENT_BUFFER.fs = MOUSE_EVENT_BUFFER.fs | MOUSE_BN3_DOWN;
	MOUSE_SAVE_FS = MOUSE_EVENT_BUFFER.fs;
	if (flag == ON)
	    while (xregs.x.bx)
	    {
		xregs.x.ax = 0x03;
		int86(0x33,&xregs,&xregs);
	    }
    }

    switch(MOUSE_V_MODE)
    {
    case(0):
    case(1):
	MOUSE_EVENT_BUFFER.row = xregs.x.dx / 16;
	MOUSE_EVENT_BUFFER.col = xregs.x.cx / 8;
	break;
    case(2):
    case(3):
    case(7):
    default: MOUSE_EVENT_BUFFER.row = xregs.x.dx / 8;
	MOUSE_EVENT_BUFFER.col = xregs.x.cx / 8;
    }
#endif

    return(MOUSE_EVENT_BUFFER.fs);
}

/****************************************************************************/
/*  RETURN RELEASE COUNTER                                                  */
/*                                                                          */
/*  Call:  mouse_release()                                                  */
/*                                                                          */
/*  Used in conjunction with mouse_read(0) function to know when the        */
/*  button is released.                                                     */
/*                                                                          */
/*  Returns:  0x00 = No Buttons Pressed                                     */
/*            0x01 = Button 1 Pressed                                       */
/*            0x02 = Button 2 Pressed                                       */
/*            0x03 = Button 3 Pressed                                       */
/*                                                                          */
/****************************************************************************/
int _fastcall mouse_release(button)
int button;
{
    int rc = 0;
    long time;
#ifndef OS2
    union REGS xregs;
#endif

    if (MOUSE_HANDLE == 0xff) return(rc);

    if (MOUSE_EVENT_BUFFER.fs == 0)
    {
	switch (button)
	{
	case 0: if (MOUSE_SAVE_FS & MOUSE_BN1_DOWN ||
		    MOUSE_SAVE_FS & MOUSE_MOTION_WITH_BN1_DOWN)
		rc = 1;
	    break;
	case 1: if (MOUSE_SAVE_FS & MOUSE_BN2_DOWN ||
		    MOUSE_SAVE_FS & MOUSE_MOTION_WITH_BN2_DOWN)
		rc = 2;
	    break;
	case 2: if (MOUSE_SAVE_FS & MOUSE_BN3_DOWN ||
		    MOUSE_SAVE_FS & MOUSE_MOTION_WITH_BN3_DOWN)
		rc = 3;
	}
	MOUSE_SAVE_FS = 0;
    }

    if (MOUSE_CLICK_SWT)
    {
#ifndef OS2
	xregs.h.ah = 0x00;
	int86(0x1a,&xregs,&xregs);
	MOUSE_DOUBLE_TIME = (((long) xregs.x.cx) << 16) | xregs.x.dx;
	if (rc)
	{
	    if ((MOUSE_DOUBLE_TIME - MOUSE_EVENT_BUFFER.time) < MOUSE_DBL_CLICK_THRESHOLD
		    && MOUSE_DOUBLE_ROW == MOUSE_EVENT_BUFFER.row)
		MOUSE_DOUBLE_CLICK = 1;
	    else
		MOUSE_DOUBLE_CLICK = 0;
	    MOUSE_EVENT_BUFFER.time = MOUSE_DOUBLE_TIME;
	    MOUSE_DOUBLE_ROW = MOUSE_EVENT_BUFFER.row;
	} else
	    MOUSE_DOUBLE_CLICK = 0;
#else
	if (rc)
	{
	    if ((MOUSE_EVENT_BUFFER.time - MOUSE_DOUBLE_TIME) < MOUSE_DBL_CLICK_THRESHOLD
		    && MOUSE_DOUBLE_ROW == MOUSE_EVENT_BUFFER.row)
		MOUSE_DOUBLE_CLICK = 1;
	    else
		MOUSE_DOUBLE_CLICK = 0;
	    MOUSE_DOUBLE_TIME = MOUSE_EVENT_BUFFER.time;
	    MOUSE_DOUBLE_ROW = MOUSE_EVENT_BUFFER.row;
	} else
	    MOUSE_DOUBLE_CLICK = 0;
#endif
    } else
	MOUSE_DOUBLE_CLICK = 1;

    return(rc);
}
/****************************************************************************/
/*  MOUSE POSITION                                                          */
/*                                                                          */
/*  Call:  mouse_position(row,col)                                          */
/*         row = integer initial row position of cursor                     */
/*         col = integer initial column position of cursor                  */
/*                                                                          */
/****************************************************************************/
void _fastcall mouse_position(row,col)
int row,col;
{
#ifndef OS2
    union REGS xregs,oregs;
#endif

    if (MOUSE_HANDLE == 0xff) return;
#ifdef OS2
    MOUSE_POS.row = row;                 /* Second Row      */
    MOUSE_POS.col = col;                 /* Column position */
    MouSetPtrPos((PTRLOC far *) &MOUSE_POS, MOUSE_HANDLE);
#else
    switch(MOUSE_V_MODE)
    {
    case(0):
    case(1):
	xregs.x.dx = row * 16;
	xregs.x.cx = col * 8;
	break;
    case(2):
    case(3):
    case(7):
    default: xregs.x.dx = row * 8;
	xregs.x.cx = col * 8;
    }

    xregs.x.ax = 0x04;
    int86(0x33,&xregs,&oregs);
#endif
}

/****************************************************************************/
/*  MOUSE Save State Buffer                             DOS VERSION ONLY    */
/*                                                                          */
/*  Call:  mouse_save_buffer()                                              */
/*                                                                          */
/****************************************************************************/
void _fastcall mouse_save_buffer()
{
#ifndef OS2
    char far *fptr = "";
    union REGS xregs;
    struct SREGS segregs;
#endif

    if (MOUSE_HANDLE == 0xff) return;

    mouse_hide();

#ifndef OS2
/*
        if (MOUSE_STATE_PTR == NULL) {
                xregs.x.ax = 0x15;
                int86(0x33,&xregs,&xregs);
                MOUSE_STATE_PTR = calloc(1,xregs.x.bx);
        }
        xregs.x.dx = (int) MOUSE_STATE_PTR;
        segregs.es = FP_SEG(fptr);
        xregs.x.ax = 0x16;
        int86x(0x33,&xregs,&xregs,&segregs);
*/
#endif
}

/****************************************************************************/
/*  MOUSE Restore State Buffer                          DOS VERSION ONLY    */
/*                                                                          */
/*  Call:  mouse_restore_buffer()                                           */
/*                                                                          */
/****************************************************************************/
void _fastcall mouse_restore_buffer()
{
#ifndef OS2
    char far *fptr = "";
    union REGS xregs;
    struct SREGS segregs;
#endif

    if (MOUSE_HANDLE == 0xff) return;
#ifdef OS2
    MouFlushQue(MOUSE_HANDLE);
    mouse_position(MOUSE_EVENT_BUFFER.row,MOUSE_EVENT_BUFFER.col);
#else
/*
        if (MOUSE_STATE_PTR != NULL) {
                xregs.x.dx = (int) MOUSE_STATE_PTR;
                segregs.es = FP_SEG(fptr);
                xregs.x.ax = 0x17;
                int86x(0x33,&xregs,&xregs,&segregs);
                free(MOUSE_STATE_PTR);
                MOUSE_STATE_PTR = NULL;
        }
*/
#endif
    mouse_show();
}

/****************************************************************************/
/*  MOUSE SET Horizonal & Vertical Limits               DOS VERSION ONLY    */
/*                                                                          */
/*  Call:  mouse_coordinates(rb,re,cb,ce)                                   */
/*         rb  = row beginning       cb = column beginning                  */
/*         re  = row end             ce = column end                        */
/*                                                                          */
/****************************************************************************/
void _fastcall mouse_coordinates(rb,re,cb,ce)
int rb,re,cb,ce;
{
#ifndef OS2
    union REGS xregs,oregs;

    if (MOUSE_HANDLE == 0xff) return;

    MOUSE_RB = rb;
    MOUSE_RE = re;
    MOUSE_CB = cb;
    MOUSE_CE = ce;

    switch(MOUSE_V_MODE)
    {
    case(0):
    case(1):
	xregs.x.cx = cb * 16;
	xregs.x.dx = ce * 16;
	break;
    case(2):
    case(3):
    case(7):
    default: xregs.x.cx = cb * 8;
	xregs.x.dx = ce * 8;
    }

    xregs.x.ax = 0x07;
    int86(0x33,&xregs,&oregs);
    xregs.x.cx = rb * 8;
    xregs.x.dx = re * 8;
    xregs.x.ax = 0x08;
    int86(0x33,&xregs,&oregs);
#endif
}

/****************************************************************************/
/*  CLOSE MOUSE                                                             */
/*                                                                          */
/*  Call:  mouse_close()                                                    */
/*                                                                          */
/****************************************************************************/
void _fastcall mouse_close()
{
    if (MOUSE_HANDLE == 0xff) return;

#ifdef OS2
    MouClose(MOUSE_HANDLE);
#else
    mouse_hide();
/*
        if (MOUSE_STATE_PTR != NULL) {
                free(MOUSE_STATE_PTR);
                MOUSE_STATE_PTR = NULL;
        }
*/
#endif

    MOUSE_HANDLE = 0xff;
}

/****************************************************************************/
/*  SHOW MOUSE                                                              */
/*                                                                          */
/*  Call:  mouse_show()                                                     */
/*                                                                          */
/*  Always hide the mouse with the mouse_hide() function before changing    */
/*  the screen.  Then use the mouse_show() function to redisplay the        */
/*  cursor.                                                                 */
/*                                                                          */
/****************************************************************************/
int _fastcall mouse_show()
{
#ifndef OS2
    union REGS xregs;
#endif

    if (MOUSE_HANDLE == 0xff) return(0);
    if (MOUSE_HIDE_COUNT)
    {
#ifndef OS2
	xregs.x.ax = 0x0001;
	int86(0x33,&xregs,&xregs); /* show cursor */
#else
	MouDrawPtr(MOUSE_HANDLE);
#endif
	MOUSE_HIDE_COUNT--;
	return(1);
    }
#ifdef DEBUG
    else
	bell();
#endif
    return(0);
}

/****************************************************************************/
/*  HIDE MOUSE                                                              */
/*                                                                          */
/*  Call:  mouse_hide()                                                     */
/*                                                                          */
/*  Always hide the mouse with the mouse_hide() function before changing    */
/*  the screen.  Then use the mouse_show() function to redisplay the        */
/*  cursor.                                                                 */
/*                                                                          */
/****************************************************************************/
int _fastcall mouse_hide()
{
#ifndef OS2
    union REGS xregs;
#else
    NOPTRRECT rptr;
    VIOMODEINFO ModData;
#endif

    if (MOUSE_HANDLE == 0xff) return(0);
    if (!MOUSE_HIDE_COUNT)
    {
#ifndef OS2
	xregs.x.ax = 0x0002;
	int86(0x33,&xregs,&xregs); /* hide cursor */
#else
	ModData.cb = sizeof(VIOMODEINFO);
	VioGetMode((VIOMODEINFO far *) &ModData,0);
	rptr.row = rptr.col = 0;
	rptr.cRow = ModData.row-1;
	rptr.cCol = ModData.col-1;
	MouRemovePtr((NOPTRRECT far *) &rptr,MOUSE_HANDLE);
#endif
	MOUSE_HIDE_COUNT++;
	return(1);
    }
#ifdef DEBUG
    else
	bell();
#endif
    return(0);
}

/****************************************************************************/
/*  CHECK KEYSTOKE AVAILABLE                                                */
/*                                                                          */
/*  Call:  mouse_ki_chk()                                                   */
/*                                                                          */
/*  Returns: 1 = Keystroke available                                        */
/*           0 = Waiting for Keystroke to be entered                        */
/*                                                                          */
/****************************************************************************/
int _fastcall mouse_ki_chk()
{
#ifdef OS2
    KBDKEYINFO CharData;

    KbdPeek((char far *) &CharData,0);
    if (CharData.fbStatus & 0x40)
	return(1);
    return(0);
#else
    if (_bios_keybrd(_KEYBRD_READY))    /* check keyboard buffer?             */
	return(1);
    return(0);
#endif
}
